﻿using System;
using System.Drawing;
using System.Windows.Forms;
using TelegramParsingLibrary;

namespace ParsingTelegramChannel.BO
{
    public class RtbExtended : RichTextBox
    {
        public bool ShowClearHistory { get; set; } = false;
        public event EventHandler ClearHistory;

        private object _locker = new object();
        private ContextMenuStrip contextMenuStrip1;
        private ToolStripMenuItem clearAllToolStripMenuItem;
        private ToolStripMenuItem clearHistoryToolStripMenuItem;

        public RtbExtended()
        {
            try
            {
                contextMenuStrip1 = new System.Windows.Forms.ContextMenuStrip();
                clearAllToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
                clearHistoryToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();

                // 
                // clearAllToolStripMenuItem
                // 
                clearAllToolStripMenuItem.Name = "clearAllToolStripMenuItem";
                clearAllToolStripMenuItem.Size = new System.Drawing.Size(147, 22);
                clearAllToolStripMenuItem.Text = "Очистить всё";
                clearAllToolStripMenuItem.Click += new System.EventHandler(clearAllToolStripMenuItem_Click);

                // 
                // clearHistoryToolStripMenuItem
                // 
                clearHistoryToolStripMenuItem.Name = "clearHistoryToolStripMenuItem";
                clearHistoryToolStripMenuItem.Size = new System.Drawing.Size(147, 22);
                clearHistoryToolStripMenuItem.Text = "Очистить историю";
                clearHistoryToolStripMenuItem.Click += new System.EventHandler(clearHistoryToolStripMenuItem_Click);

                // 
                // contextMenuStrip1
                // 
                contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            clearAllToolStripMenuItem, clearHistoryToolStripMenuItem});
                contextMenuStrip1.Name = "contextMenuStrip1";
                contextMenuStrip1.Size = new System.Drawing.Size(148, 26);

                MouseUp += RtbExtended_MouseUp;
                this.VisibleChanged += RtbExtended_VisibleChanged;
            }
            catch(Exception ex) { Logger.WriteError(ex); }
        }

        private void RtbExtended_VisibleChanged(object sender, EventArgs e)
        {
            if (!ShowClearHistory && contextMenuStrip1.Items.Contains(clearHistoryToolStripMenuItem))
            {
                contextMenuStrip1.Items.Remove(clearHistoryToolStripMenuItem);
            }
            else if (ShowClearHistory && !contextMenuStrip1.Items.Contains(clearHistoryToolStripMenuItem))
            {
                contextMenuStrip1.Items.Add(clearHistoryToolStripMenuItem);
            }
        }

        private void clearHistoryToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ClearHistory?.Invoke(this, EventArgs.Empty);
        }

        private void RtbExtended_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                contextMenuStrip1.Show(this, e.X, e.Y);
            }
        }

        private void clearAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Clear();
        }

        public void NewMessage(string text, bool addTimeStamp = true)
        {
            if (InvokeRequired)
            {
                Invoke((MethodInvoker)delegate () { NewMessage(text, addTimeStamp); });
                return;
            }

            lock (_locker)
            {
                if (text.Length > 1)
                {
                    if (addTimeStamp)
                    {
                        AppendText(DateTime.Now.ToString("HH:mm:ss "));
                    }

                    for (int i = 0; i < text.Length; i++)
                    {
                        if (text[i].Equals('<') &&
                            (text[i + 1].Equals('r') || text[i + 1].Equals('/') || text[i + 1].Equals('g') || text[i + 1].Equals('b'))) //start teg
                        {
                            i++;

                            if (text[i].Equals('r')) //start red
                            {
                                SelectionColor = Color.Red;
                                i++;
                            }
                            else if (text[i].Equals('/') && text[i + 1].Equals('r')) //end red
                            {
                                SelectionColor = Color.Black;
                                i += 2;
                            }
                            else if (text[i].Equals('g')) //start green
                            {
                                SelectionColor = Color.Green;
                                i++;
                            }
                            else if (text[i].Equals('/') && text[i + 1].Equals('g')) //end green
                            {
                                SelectionColor = Color.Black;
                                i += 2;
                            }
                            else if (text[i].Equals('b')) //start bold
                            {
                                SelectionFont = new Font(Font, FontStyle.Bold);
                                i++;
                            }
                            else if (text[i].Equals('/') && text[i + 1].Equals('b')) //end bold
                            {
                                SelectionFont = new Font(Font, FontStyle.Regular);
                                i += 2;
                            }
                        }
                        else
                        {
                            AppendText(text.Substring(i, 1));
                        }
                    }

                    AppendText("\n");
                }
                else
                {
                    AppendText(text);
                }

                ScrollToCaret();
            }
        }
    }
}
