﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace TelegramParsingLibrary
{
    public class ProxyTcpClient
    {
        public TcpClient GetClient(string targetHost, int targetPort, string httpProxyHost, int httpProxyPort, string proxyUserName, string proxyPassword)
        {
            const BindingFlags Flags = BindingFlags.NonPublic | BindingFlags.Instance;

            Uri proxyUri = new UriBuilder
            {
                Scheme = Uri.UriSchemeHttp,
                Host = httpProxyHost,
                Port = httpProxyPort
            }.Uri;

            Uri targetUri = new UriBuilder
            {
                Scheme = targetPort == 443 ? Uri.UriSchemeHttps : Uri.UriSchemeHttp,
                Host = targetHost,
                Port = targetPort
            }.Uri;

            WebProxy webProxy = new WebProxy(proxyUri, true);
            webProxy.Credentials = new NetworkCredential(proxyUserName, proxyPassword);
            WebRequest request = WebRequest.Create(targetUri);
            request.Proxy = webProxy;
            request.Method = "CONNECT";
            HttpWebResponse response = (HttpWebResponse)request.GetResponse();
            Stream responseStream = response.GetResponseStream();
            Type responseType = responseStream.GetType();
            PropertyInfo connectionProperty = responseType.GetProperty("Connection", Flags);
            var connection = connectionProperty.GetValue(responseStream, null);
            Type connectionType = connection.GetType();
            PropertyInfo networkStreamProperty = connectionType.GetProperty("NetworkStream", Flags);
            NetworkStream networkStream = (NetworkStream)networkStreamProperty.GetValue(connection, null);
            Type nsType = networkStream.GetType();
            PropertyInfo socketProperty = nsType.GetProperty("Socket", Flags);
            Socket socket = (Socket)socketProperty.GetValue(networkStream, null);

            return new TcpClient { Client = socket };
        }

        public TcpClient ConnectProxy(string host, int port, string httpProxyHost, int httpProxyPort, string proxyUserName, string proxyUserPass)
        {
            var url = "http://" + host + ":" + port;
            var urlProxy = "http://" + httpProxyHost + ":" + httpProxyPort;
            var proxyUrl = WebRequest.DefaultWebProxy.GetProxy(new Uri(url));
            WebResponse response = null;
            var tentativas = 10;

            while (tentativas >= 0)
            {
                var request = (HttpWebRequest)WebRequest.Create(proxyUrl);
                request.KeepAlive = true;

                if (!string.IsNullOrEmpty(httpProxyHost) && httpProxyPort > 0)
                {
                    ICredentials credentials = null;
                    if (!string.IsNullOrEmpty(proxyUserName))
                    {
                        credentials = new NetworkCredential(proxyUserName, proxyUserPass);
                    }
                    var webProxy = new WebProxy(urlProxy, true, null, credentials);
                    request.Proxy = webProxy;
                }

                request.Method = "CONNECT";
                request.Timeout = 1000;

                tentativas--;
                try
                {
                    response = request.GetResponse();
                    break;
                }
                catch (Exception ex)
                {
                    Logger.WriteError(ex);
                }
            }

            var responseStream = response.GetResponseStream();

            const BindingFlags Flags = BindingFlags.NonPublic | BindingFlags.Instance;

            var rsType = responseStream.GetType();
            var connectionProperty = rsType.GetProperty("Connection", Flags);

            var connection = connectionProperty.GetValue(responseStream, null);
            var connectionType = connection.GetType();
            var networkStreamProperty = connectionType.GetProperty("NetworkStream", Flags);

            var networkStream = networkStreamProperty.GetValue(connection, null);
            var nsType = networkStream.GetType();
            var socketProperty = nsType.GetProperty("Socket", Flags);
            var socket = (Socket)socketProperty.GetValue(networkStream, null);

            return new TcpClient { Client = socket };
        }
    }
}
