﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using TeleSharp.TL;
using TeleSharp.TL.Messages;
using TLSharp.Core;

namespace TelegramParsingLibrary
{
    public class ChannelInfo
    {
        public TLChannel Channel { get; set; }
        public TeleSharp.TL.Messages.TLChatFull ChatFull { get; set; }
        public List<TLUser> Users { get; set; } = new List<TLUser>();

        private DateTime _dateCreated;
        public DateTime DateCreated
        {
            get { return _dateCreated; }
            set
            {
            //    _dateCreated = new DateTime(1970, 1, 1, 0, 0, 0, 0, DateTimeKind.Utc).AddSeconds(Channel.date).ToLocalTime();
            }
        }
    }
    public class TelegramHelper
    {
        private int _apiId;
        private string _apiHash;
        private string _phoneNumber;
        private Action<string> _message;
        private TelegramClient _client;
        private string _hash;
        private TLUser _user;
        private int _lastSignalMessageDate;

        private string _proxyHost;
        private int _proxyPort;
        private string _proxyUserName;
        private string _proxyUserPass;

        public bool Connected { get; private set; }

        public TelegramHelper(int apiId, string apiHash, string phoneNumber, Action<string> message, string proxyHost, int proxyPort,
            string proxyUserName, string proxyUserPass)
        {
            _apiId = apiId;
            _apiHash = apiHash;
            _phoneNumber = phoneNumber;
            _message = message;

            if (!string.IsNullOrEmpty(proxyHost) && proxyPort > 0)
            {
                _proxyHost = proxyHost;
                _proxyPort = proxyPort;
                _proxyUserName = proxyUserName;
                _proxyUserPass = proxyUserPass;
            }
        }

        public TcpClient _connect(string ip, int port)
        {
            ProxyTcpClient client = new ProxyTcpClient();
            return client.ConnectProxy(ip, port, _proxyHost, _proxyPort, _proxyUserName, _proxyUserPass);
        }

        public async Task<bool> Connect(bool noretry = false)
        {
            try
            {
                if (_client == null)
                {
                    _client = new TelegramClient(_apiId, _apiHash, handler: _connect);
                }

                await _client.ConnectAsync();

                if (!_client.IsUserAuthorized())
                   // if (!_client.IsConnected)
                    {
                    newMessage("Отсылка кода по Телеграм!");
                    _hash = await _client.SendCodeRequestAsync(_phoneNumber);
                    Connected = false;
                    return true;
                }
                else
                {
                    Connected = await _client.IsPhoneRegisteredAsync(_phoneNumber);

                    if (Connected)
                    {
                        newMessage("<g>Уже подключены к Телеграм!</g>");
                    }
                }
            }
            catch (OutOfMemoryException ex)
            {
                Logger.WriteError(ex);
                _client = null;
                return await Connect(true);
            }
            catch (Exception ex)
            {
                Logger.WriteError(ex);
                try
                {
                    if (ex.Message.Equals("AUTH_RESTART") && !noretry)
                    {
                        return await Connect(true);
                    }
                }
                catch (Exception exe) { Logger.WriteError(exe); }
            }
            return false;
        }

        public async Task<List<Tuple<DateTime, string, string>>> GetNewPosts(string channelName)
        {
            if (_client == null)
            {
                return null;
            }

            try
            {
                if (_client.IsUserAuthorized())
                {
                    if (channelName != null && channelName.Length > 0)
                    {
                        dynamic dialogs = null;

                        TLAbsDialogs dias = await _client.GetUserDialogsAsync();
                        if (dias is TLDialogsSlice)
                        {
                            dialogs = (TLDialogsSlice)dias;
                        }
                        else if (dias is TLDialogs)
                        {
                            dialogs = (TLDialogs)dias;
                        }

                        if (dialogs != null)
                        {
                            List<TLChannel> signalChannel = new List<TLChannel>();

                            foreach (var c in ((TLVector<TLAbsChat>)dialogs.Chats).ToList())
                            {
                                if (c is TLChannel && !string.IsNullOrEmpty(((TLChannel)c).Username) && channelName.Equals(((TLChannel)c).Username))
                                {
                                    signalChannel.Add((TLChannel)c);
                                }
                            }

                            if (signalChannel.Count > 0)
                            {
                                newMessage($"<g>Проверяем сигнальный канал<b> {channelName}</b>!</g>");
                            }
                            else
                            {
                                newMessage($"<r>Не найдены каналы для проверки сигналов!</r>");
                                return null;
                            }

                            if (signalChannel.Count == 0)
                            {
                                return null;
                            }

                            var lst = from q in ((TLVector<TLAbsMessage>)dialogs.Messages).ToList()
                                      where q is TLMessage
                                            && ((TLMessage)q).Date > _lastSignalMessageDate
                                            && ((TLMessage)q).ToId is TLPeerChannel
                                      orderby ((TLMessage)q).Date
                                      select ((TLMessage)q);

                            var result = new List<Tuple<DateTime, string, string>>();

                            foreach (var m in lst)
                            {
                                if (m != null && m.ToId != null)
                                {
                                    var cc = signalChannel.Find(c => c.Id.Equals(((TLPeerChannel)m.ToId).ChannelId));
                                    if (cc != null && _lastSignalMessageDate < ((TLMessage)m).Date)
                                    {
                                        result.Add(new Tuple<DateTime, string, string>(m.Date.Unix2DateTime(), m.Message, $"https://t.me/{cc.Username}/{m.Id}"));
                                        //newMessage($"<g><b>{cc.Username}:</g></b> " + ((TLMessage)m).Message);

                                        _lastSignalMessageDate = Math.Max(_lastSignalMessageDate, ((TLMessage)m).Date);
                                    }
                                }
                            }
                            return result;
                        }
                        else
                        {
                            newMessage($"<r>Message not a `TLDialogs` and `TLDialogsSlice`! It is `{dias.GetType()}`</r>");
                        }
                    }
                    else
                    {
                        newMessage($"<r>Проверьте настройки каналов!</r>");
                        return null;
                    }
                }
                else
                {
                    newMessage("<r>User not connected!</r>");
                    Connected = false;
                }
            }
            catch (Exception ex) { Logger.WriteError(ex); }
            return null;
        }

        public async Task<List<Tuple<DateTime, string, string>>> GetLastPosts(string channelName, int count)
        {
            if (_client == null)
            {
                return null;
            }

            try
            {
                if (_client.IsUserAuthorized())
                {
                    if (channelName != null && channelName.Length > 0)
                    {
                        dynamic dialogs = null;

                        TLAbsDialogs dias = await _client.GetUserDialogsAsync();
                        if (dias is TLDialogsSlice)
                        {
                            dialogs = (TLDialogsSlice)dias;
                        }
                        else if (dias is TLDialogs)
                        {
                            dialogs = (TLDialogs)dias;
                        }

                        if (dialogs != null)
                        {
                            List<TLChannel> signalChannel = new List<TLChannel>();

                            foreach (var c in ((TLVector<TLAbsChat>)dialogs.Chats).ToList())
                            {
                                if (c is TLChannel && !string.IsNullOrEmpty(((TLChannel)c).Username) && channelName.Equals(((TLChannel)c).Username))
                                {
                                    signalChannel.Add((TLChannel)c);
                                }
                            }

                            if (signalChannel.Count > 0)
                            {
                                newMessage($"<g>Проверяем сигнальный канал<b> {channelName}</b>!</g>");
                            }
                            else
                            {
                                newMessage($"<r>Не найдены каналы для проверки сигналов!</r>");
                                return null;
                            }
                            var target = new TLInputPeerChannel() { ChannelId = signalChannel[0].Id, AccessHash = (long)signalChannel[0].AccessHash };
                            TLChannelMessages hist = (TLChannelMessages)await _client.GetHistoryAsync(target, 0, -1, count);
                            int par = 0;
                           // int offset = 0;
                            var chan = await _client.SendRequestAsync<TeleSharp.TL.Messages.TLChatFull>(new TeleSharp.TL.Channels.TLRequestGetFullChannel()
                            {
                                Channel = new TLInputChannel()
                                { ChannelId = signalChannel[0].Id, AccessHash = (long)signalChannel[0].AccessHash }
                            });
                            var coun = chan.FullChat; //  FullChat.Deserialize.
                                                      //  TLInputPeerChannel inputPeer = new TLInputPeerChannel()
                                                      //  { ChannelId = signalChannel[0].Id, AccessHash = (long)signalChannel[0].AccessHash };
                                                      //List<TeleSharp.TL.Channels.TLChannelParticipants> userChannel = new List<TeleSharp.TL.Channels.TLChannelParticipants>();
                                                      //while (true)
                                                      //{

                            //    var request = new TeleSharp.TL.Channels.TLRequestGetParticipants
                            //    {
                            //        Offset = offset,
                            //        Limit = 200,
                            //        Channel = new TLInputChannel()
                            //        { ChannelId = signalChannel[0].Id, AccessHash = (long)signalChannel[0].AccessHash },
                            //        Filter = new TLChannelParticipantsRecent()
                            //    };
                            //    TeleSharp.TL.Channels.TLChannelParticipants found = await _client.SendRequestAsync<TeleSharp.TL.Channels.TLChannelParticipants>(request);
                            //    par = par + found.Participants.Count;
                            //    userChannel.Add((TeleSharp.TL.Channels.TLChannelParticipants)found);
                            //    offset = offset + 1;
                            //};

                            var result2 = new ChannelInfo();
                           // var dialogs2 = (TLDialogs)await _client.GetUserDialogsAsync();
                          //  var main = dialogs2.Chats.lists.Where(c => c.GetType() == typeof(TLChannel))
                          //              .Cast<TLChannel>()
                         //               .FirstOrDefault(c => c.title == (groupName));
                            //var req = new TeleSharp.TL.Channels.TLRequestGetFullChannel()
                            //{
                            //    Сhannel = new TLInputChannel() { AccessHash = main.access_hash.Value, channel_id = main.id }
                            //};
                            var res = await _client.SendRequestAsync<TeleSharp.TL.Messages.TLChatFull>(new TeleSharp.TL.Channels.TLRequestGetFullChannel()
                            {
                                Channel = new TLInputChannel()
                                { ChannelId = signalChannel[0].Id, AccessHash = (long)signalChannel[0].AccessHash }
                            });
                           // var res = await _client.SendRequestAsync<TeleSharp.TL.Messages.TLChatFull>(req);

                            //we have to do this in slices
                            var offset = 0;
                          //  result2.Channel = main;
                            result2.ChatFull = res;
                            int countCan = (res.FullChat as TLChannelFull).ParticipantsCount.Value;
                           //     while (offset < (res.FullChat as TLChannelFull).ParticipantsCount)
                           //// while (result2.Users.Count < (res.FullChat as TLChannelFull).ParticipantsCount)
                           // {
                           //     var pReq = new TeleSharp.TL.Channels.TLRequestGetParticipants
                           //     {
                           //         Offset = offset,
                           //        // Q = "M";
                           //         Limit = 200,
                           //         Channel = new TLInputChannel()
                           //         { ChannelId = signalChannel[0].Id, AccessHash = (long)signalChannel[0].AccessHash },
                           //         Filter = new TLChannelParticipantsRecent()
                           //     };
                           //     var pRes = await _client.SendRequestAsync<TeleSharp.TL.Channels.TLChannelParticipants>(pReq);
                           //     result2.Users.AddRange(pRes.Users.Cast<TLUser>());
                           //     offset += 200;
                           //     await Task.Delay(500);
                           // }

                           
                            //List<string> username = new List<string>();
                            var username = new List<Tuple<DateTime, string, string>>();
                            var mst = from q in result2.Users.ToList()
                                      where q is TLUser
                                      //      && ((TLUser)q).Date > _lastSignalMessageDate
                                      //      && ((TLUser)q).ToId is TLPeerChannel
                                      //orderby ((TLUser)q).Date
                                      select ((TLUser)q);

            //                foreach (var m in mst)
            //                {
            //                    if (m != null)
            //                    {
                                   // var cc = signalChannel.Find(c => c.Id.Equals(((TLPeerChannel)m.ToId).ChannelId));
                                   // if (cc != null && _lastSignalMessageDate < ((TLMessage)m).Date)
                                   // {
                            //            username.Add(new Tuple<DateTime, string, string>(DateTime.UtcNow, m.Id.ToString(), m.Username));
                                      
                                //newMessage($"<g><b>{cc.Username}:</g></b> " + ((TLMessage)m).Message);

                                    //    _lastSignalMessageDate = Math.Max(_lastSignalMessageDate, ((TLMessage)m).Date);
                                  //  }
             //                   }
             //               }
            //                return username;

                            //return result2;

                            var lst = from q in hist.Messages.ToList()
                                      where q is TLMessage
                         //                   && ((TLMessage)q).Date > _lastSignalMessageDate
                                            && ((TLMessage)q).ToId is TLPeerChannel
                                      orderby ((TLMessage)q).Date
                                      select ((TLMessage)q);
                            int unixTime = (int)(DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds;
                            
                            // string fileName = @System.IO.Directory.GetCurrentDirectory() + "\\" + channelName + "\\" + channelName +"TT"+ unixTime.ToString() + ".txt";
                            string fileName = @System.IO.Directory.GetCurrentDirectory() + "\\" + channelName + "\\" + channelName + ".txt";

                            string fileName2 = @System.IO.Directory.GetCurrentDirectory() + "\\" + channelName;

                            //string fileName = @System.IO.Directory.GetCurrentDirectory() + "\\" + channelName + "TT" + unixTime.ToString() + ".txt";
                            System.IO.DirectoryInfo di = System.IO.Directory.CreateDirectory(fileName2);
                            //   string fileName = @"d:\pp\"+ channelName+".txt";
                            //  System.IO.FileStream aFile = new System.IO.FileStream(fileName, System.IO.FileMode.OpenOrCreate);
                            System.IO.FileStream aFile = new System.IO.FileStream(fileName, System.IO.FileMode.Create);
                            System.IO.StreamWriter sw = new System.IO.StreamWriter(aFile);
                            aFile.Seek(0, System.IO.SeekOrigin.End);
                            var result = new List<Tuple<DateTime, string, string>>();
                            string text;
                            foreach (var m in lst)
                            {
                                if (m != null && m.ToId != null)
                                {
                                    //     var cc = signalChannel.Find(c => c.Id.Equals(((TLPeerChannel)m.ToId).ChannelId));
                                    //    if (cc != null && _lastSignalMessageDate < ((TLMessage)m).Date)
                                    var cc = signalChannel.Find(c => c.Id.Equals(((TLPeerChannel)m.ToId).ChannelId));
                                      if (cc != null )
                                    {
                                        //Создадим обьект класса XmlSerializer, для выполнения сериализации обьекта(в скобачках тип сеарилизованых данных)
                                        

                                     //   text = unixTime.ToString() +";"+ DateTime.Now.ToString("dd/MM/yyyy HH:mm:ss") + ";" + m.Date.Unix2DateTime() + ";" + countCan.ToString()+ ";" + m.Views.ToString() + ";" + $"https://t.me/{cc.Username}/{m.Id}";
                                        text = m.Date.Unix2DateTime() + ";" + $"https://t.me/{cc.Username}/{m.Id}" + ";" + unixTime.ToString() + ";" + DateTime.Now.ToString("dd/MM/yyyy HH:mm:ss") + ";" +  countCan.ToString() + ";" + m.Views.ToString() + ";" + m.Message.ToString();
                                        sw.WriteLine(text);
                                      //  sw.WriteLine("abc");
                                        
                                         result.Add(new Tuple<DateTime, string, string>(m.Date.Unix2DateTime(), m.Views.ToString(), $"https://t.me/{cc.Username}/{m.Id}"));
                                        //    result.Add(new Tuple<DateTime, string, string>(m.Date.Unix2DateTime(), m.Message, $"https://t.me/{cc.Username}/{m.Id}"));
                                        //newMessage($"<g><b>{cc.Username}:</g></b> " + ((TLMessage)m).Message);

                                        //      _lastSignalMessageDate = Math.Max(_lastSignalMessageDate, ((TLMessage)m).Date);
                                    }
                                }
                            }
                            sw.Close();
                            return result;
                            
                        }
                        else
                        {
                            newMessage($"<r>Message not a `TLDialogs` and `TLDialogsSlice`! It is `{dias.GetType()}`</r>");
                        }
                    }
                    else
                    {
                        newMessage($"<r>Проверьте настройки каналов!</r>");
                        return null;
                    }
                }
                else
                {
                    newMessage("<r>User not connected!</r>");
                    Connected = false;
                }
            }
            catch (Exception ex) { Logger.WriteError(ex); }
            return null;
        }

        public async Task<bool> SignInWithCode(string phone_code, Func<string> getPassword)
        {
            try
            {
                newMessage("Проверка кода по Телеграм!");
                if (!_client.IsUserAuthorized())
                {
                    _user = await _client.MakeAuthAsync(_phoneNumber, _hash, phone_code);
                    if (_user != null && _client.IsUserAuthorized())
                    {
                        newMessage("Подключены к Телеграм!");
                        Connected = true;
                        return true;
                    }
                }
                else
                {
                    Connected = true;
                    return true;
                }
            }
            catch (CloudPasswordNeededException ex)
            {
                Logger.WriteError(ex);
                try
                {
                    newMessage("<r>Нужна авторизация через форму!!!</r>");
                    Logger.Write("Нужна авторизация через форму!!!");
                    var password_str = getPassword();
                    if (!string.IsNullOrEmpty(password_str))
                    {
                        var password = await _client.GetPasswordSetting();

                        _user = await _client.MakeAuthWithPasswordAsync(password, password_str);

                        if (_user != null && _client.IsUserAuthorized())
                        {
                            newMessage("Подключены к Телеграм!");
                            Connected = true;
                            return true;
                        }
                    }
                }
                catch (Exception exe) { Logger.WriteError(exe); }
            }
            catch (Exception ex) { Logger.WriteError(ex); }
            return false;
        }

        private void newMessage(string v)
        {
            if (_message != null)
            {
                _message.Invoke(v);
            }
        }
    }
}
